/*-------------------------------------------------------------------------
 * Copyright (c) 2000-2007 Kenneth W. Sodemann (ken.sodemann@gmail.com)
 * Copyright (c) 2014, Alex Panin (alex@linuxonly.ru) and Alexey Loginov (loginov.alex.valer@gmail.com)
 *-------------------------------------------------------------------------
 * cards
 *
 * Synopsis:
 *   routines for handling the MathWar cards
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to
 * Free Software Foundation, Inc.
 * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 *-------------------------------------------------------------------------
 */

#ifdef HAVE_CONFIG_H
   #include <config.h>
#endif

#include <gtk/gtk.h>
#include <glib/gi18n.h>

#include "cards.h"

/*
 * Cards values are zero through twenty.  There are twenty-one number
 * cards, and two back cards.  The index is offset by two card backs
 * (IOW, card back one is @ 0, card back two is @ 1, card0 is @ 2, 
 * card1 is @ 3, ..., card20 is @22).
 */
#define MIN_CARD         0
#define MAX_CARD        20
#define NUMCARDS        23
#define IDX_OFFSET       2

#define SHADOW_SIZE      3

#define FILENAME_LENGTH 32

/*
 * Module variables
 */
static GdkPixbuf *deck[NUMCARDS] = {NULL};
static gboolean deckloaded = FALSE;

static GdkPixbuf *load_card_pixbuf(guint index)
{
	GdkPixbuf *pixbuf;
	gchar *filename;
	gchar *filepath;
	GError *error;
	
	pixbuf = NULL;
	
	if (index < NUMCARDS) {
		/*Form filename*/
		filename = g_alloca(FILENAME_LENGTH);
		memset(filename, 0, FILENAME_LENGTH);
		if (index < IDX_OFFSET) {
			/*Back sides*/
			g_snprintf(filename, FILENAME_LENGTH, "card_back%u.png", index+1);
		} else {
			/*Front sides*/
			g_snprintf(filename, FILENAME_LENGTH, "card_%u.png", index-IDX_OFFSET);
		}
		/*Full path*/
		filepath = g_build_filename(PACKAGE_DATA_DIR, "pixmaps", filename, NULL);
		/*Load pixbuf*/
		error = NULL;
		pixbuf = gdk_pixbuf_new_from_file(filepath, &error);
		if (pixbuf == NULL) {
			g_warning("Failed to load pixbuf %s: %s", filepath, error->message);
			g_error_free(error);
		}
		/*Free full path*/
		g_free(filepath);
	}
	
	return pixbuf;
}

gboolean load_deck(void)
{
	gboolean ret = TRUE;
	gint i;
	
	if (!deckloaded) {
		ret = TRUE;
		/*Load pixbufs*/
		for (i = 0; i < NUMCARDS; i++) {
			deck[i] = load_card_pixbuf(i);
			ret = ret && (deck[i] != NULL);
		}
		/*Free all pixbufs if some not loaded properly*/
		if (!ret) {
			for (i = 0; i < NUMCARDS; i++) {
				if (deck[i] != NULL) {
					g_object_unref(deck[i]);
				}
			}
		}
		deckloaded = ret;
	}
	
	return ret;
}

void destroy_deck(void)
{
	gint i;
	
	if (deckloaded) {
		/*Free all pixbufs*/
		for (i = 0; i < NUMCARDS; i++) {
			if (deck[i] != NULL) {
				g_object_unref(deck[i]);
			}
		}
		deckloaded = FALSE;
	}
}


void
draw_card(cairo_t *cr, gint card, gint x, gint y)
{
   g_return_if_fail(card <= MAX_CARD);
   g_return_if_fail(card >= CARD_BACK_TWO);
   
   cairo_set_source_rgb(cr, 0.0, 0.0, 0.0);
   cairo_set_line_width(cr, 1);
   cairo_rectangle(cr, x + SHADOW_SIZE, y + SHADOW_SIZE, CARD_WIDTH, CARD_HEIGHT);
   cairo_stroke_preserve(cr);
   cairo_fill(cr);
   
   gdk_cairo_set_source_pixbuf(cr, (const GdkPixbuf *)deck[card + IDX_OFFSET], (gdouble)x, (gdouble)y);
   cairo_paint(cr);
}
