/*-------------------------------------------------------------------------
 * Copyright (c) 2000-2007 Kenneth W. Sodemann (ken.sodemann@gmail.com)
 * Copyright (c) 2014, Alex Panin (alex@linuxonly.ru) and Alexey Loginov (loginov.alex.valer@gmail.com)
 *-------------------------------------------------------------------------
 * draw
 *
 * Synopsis:
 *   routines used to draw the game board
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to
 * Free Software Foundation, Inc.
 * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 *-------------------------------------------------------------------------
 */
#ifdef HAVE_CONFIG_H
    #include <config.h>
#endif

#include <gtk/gtk.h>
#include <glib/gi18n.h>

#include "cards.h"
#include "draw.h"
#include "ini_defs.h"
#include "mainwin.h"

/*
 * The score area height is set in the init_draw_objects() function
 * after we load the font that we are going to use.  I supposed this 
 * could change mid run if we ever let the user set the font.  The value
 * set here is just an estimate.
 */
#define TEXT_Y_PADDING   5
#define TEXT_X_PADDING   5
#define TEXT_INDENT      30
static gint score_area_height = 50;

/*
 * Define how the cards and opers line up in that area
 */
#define LINE_WIDTH       5
#define SIGN_SQR        20
#define CARD_X_PAD      30
#define CARD_Y_PAD      20
#define CARD_AREA_Y     (score_area_height)
#define CARD_Y          (CARD_AREA_Y) + (CARD_Y_PAD) 
#define CARD_ONE_X      (CARD_X_PAD)
#define CARD_TWO_X      ((CARD_ONE_X) + (CARD_WIDTH) + (CARD_X_PAD) + \
                         (SIGN_SQR) + (CARD_X_PAD))
#define OPER_X          ((CARD_ONE_X) + (CARD_WIDTH) + (CARD_X_PAD))
#define EQUAL_X         ((CARD_TWO_X) + (CARD_WIDTH) + (CARD_X_PAD))
#define OPER_Y          ((CARD_Y) + ((CARD_HEIGHT) / 2) - ((SIGN_SQR) / 2))
#define CARD_AREA_HEIGHT  ((CARD_Y_PAD) + (CARD_HEIGHT) + (CARD_Y_PAD))

#define FEEDBACK_AREA_Y ((CARD_AREA_Y) + (CARD_AREA_HEIGHT))
static gint feedback_area_height = 50;   
/*
 * Module variables
 */
static cairo_surface_t *surface = NULL;
static cairo_t *cairo = NULL;
static cairo_surface_t *bgsurface = NULL;
static cairo_t *bgcairo = NULL;

static void
draw_plus_sign(gint x, gint y)
{
   cairo_move_to(cairo, x + (SIGN_SQR / 2), y);
   cairo_line_to(cairo, x + (SIGN_SQR / 2), y + SIGN_SQR);
   
   cairo_move_to(cairo, x, y + (SIGN_SQR / 2));
   cairo_line_to(cairo, x + SIGN_SQR, y + (SIGN_SQR / 2));
   
   cairo_stroke(cairo);
}


static void
draw_mult_sign(gint x, gint y)
{
   cairo_move_to(cairo, x, y);
   cairo_line_to(cairo, x + SIGN_SQR, y + SIGN_SQR);
   
   cairo_move_to(cairo, x, y + SIGN_SQR);
   cairo_line_to(cairo, x + SIGN_SQR, y);
   
   cairo_stroke(cairo);
}


static void
draw_minus_sign(gint x, gint y)
{
   cairo_move_to(cairo, x, y + (SIGN_SQR / 2));
   cairo_line_to(cairo, x + SIGN_SQR, y + (SIGN_SQR / 2));

   cairo_stroke(cairo);
}


static void
draw_equal_sign(gint x, gint y)
{
   cairo_move_to(cairo, x, y + (SIGN_SQR / 4));
   cairo_line_to(cairo, x + (SIGN_SQR), y + (SIGN_SQR / 4));
   
   cairo_move_to(cairo, x, y + (SIGN_SQR * 3 / 4));
   cairo_line_to(cairo, x + (SIGN_SQR), y + (SIGN_SQR * 3 / 4));
   
   cairo_stroke(cairo);
}


static void
draw_scoreboard (const player_object *p1, const player_object *p2)
{
   gchar *fstr, *scorestr;
   PangoLayout *layout;
   PangoFontDescription *desc;
   gint width, height;
   gint x, y, player1_y, width1, width2;

   g_assert(score_area_height > 0);
   
   cairo_set_source_surface(cairo, bgsurface, 0, 0);
   cairo_rectangle(cairo, 0, 0, board_width(), score_area_height);
   cairo_fill(cairo);
   
   setup_text_gc();
   
   fstr = ini_get_string(FONTS, TEXT_FONT, DEF_TEXT_FONT);
   desc = pango_font_description_from_string(fstr);
   g_free(fstr);
   
   layout = pango_cairo_create_layout(cairo);
   pango_layout_set_font_description(layout, desc);
   pango_layout_set_text(layout, _("Scores"), -1);
   pango_layout_get_pixel_size(layout, &width, &height);
   x = TEXT_X_PADDING;
   y = TEXT_Y_PADDING;
   cairo_translate(cairo, x, y);
   pango_cairo_update_layout(cairo, layout);
   pango_cairo_show_layout(cairo, layout);
   cairo_translate(cairo, -x, -y);
   g_object_unref(layout);
   
   layout = pango_cairo_create_layout(cairo);
   pango_layout_set_font_description(layout, desc);
   pango_layout_set_text(layout, get_player_name(p1), -1);
   pango_layout_get_pixel_size(layout, &width1, &height);
   x = 3 * TEXT_X_PADDING;
   y += TEXT_Y_PADDING + height;
   player1_y = y;
   cairo_translate(cairo, x, y);
   pango_cairo_update_layout(cairo, layout);
   pango_cairo_show_layout(cairo, layout);
   cairo_translate(cairo, -x, -y);
   g_object_unref(layout);
   
   layout = pango_cairo_create_layout(cairo);
   pango_layout_set_font_description(layout, desc);
   pango_layout_set_text(layout, get_player_name(p2), -1);
   pango_layout_get_pixel_size(layout, &width2, &height);
   x = 3 * TEXT_X_PADDING;
   y += TEXT_Y_PADDING + height;
   cairo_translate(cairo, x, y);
   pango_cairo_update_layout(cairo, layout);
   pango_cairo_show_layout(cairo, layout);
   cairo_translate(cairo, -x, -y);
   g_object_unref(layout);
   
   width1 = (width1 > width2) ? width1 : width2;
   width1 += 6 * TEXT_X_PADDING;
   
   scorestr = g_strdup_printf("%d", get_player_score(p1));
   layout = pango_cairo_create_layout(cairo);
   pango_layout_set_font_description(layout, desc);
   pango_layout_set_text(layout, scorestr, -1);
   pango_layout_get_pixel_size(layout, &width2, &height);
   cairo_translate(cairo, width1, player1_y);
   pango_cairo_update_layout(cairo, layout);
   pango_cairo_show_layout(cairo, layout);
   cairo_translate(cairo, -width1, -player1_y);
   g_object_unref(layout);
   g_free(scorestr);
   
   scorestr = g_strdup_printf("%d", get_player_score(p2));
   layout = pango_cairo_create_layout(cairo);
   pango_layout_set_font_description(layout, desc);
   pango_layout_set_text(layout, scorestr, -1);
   pango_layout_get_pixel_size(layout, &width2, &height);
   cairo_translate(cairo, width1, y);
   pango_cairo_update_layout(cairo, layout);
   pango_cairo_show_layout(cairo, layout);
   cairo_translate(cairo, -width1, -y);
   g_object_unref(layout);
   g_free(scorestr);
   
   pango_font_description_free(desc);
   
   g_idle_add(update_drawing_area, NULL);
}

gint
board_height (void)
{
   return (score_area_height + CARD_AREA_HEIGHT + feedback_area_height);
}

gint
board_width (void)
{
   /*
    * The position of the equal sign + the width of the equal sign,
    * plus some padding.
    */
   return (EQUAL_X + SIGN_SQR + CARD_X_PAD + CARD_WIDTH);
}


void
draw_cards_up (gint card_one_val, gint card_two_val, operator_type operator)
{
   cairo_set_source_surface(cairo, bgsurface, 0, 0);
   cairo_rectangle(cairo, 0, CARD_AREA_Y, board_width(), CARD_AREA_HEIGHT);
   cairo_fill(cairo);
   
   draw_card(cairo, card_one_val, CARD_ONE_X, CARD_Y);
   draw_card(cairo, card_two_val, CARD_TWO_X, CARD_Y);
   
   setup_text_gc();
   
   switch (operator)
      {
      case SUBTRACTION:
         draw_minus_sign(OPER_X, OPER_Y);
         break;

      case ADDITION:
         draw_plus_sign(OPER_X, OPER_Y);
         break;

      case MULTIPLICATION:
         draw_mult_sign(OPER_X, OPER_Y);
         break;

      default:
         g_assert_not_reached();
         break;
      }

   draw_equal_sign(EQUAL_X, OPER_Y);
      
   g_idle_add(update_drawing_area, NULL);
}


void
draw_cards_down (void)
{
   /*
    * Just draw the cards "UP" using the back...
    */
   draw_cards_up (CARD_BACK_ONE, CARD_BACK_TWO, ADDITION);
}


void
clear_message_area (void)
{
   cairo_set_source_surface(cairo, bgsurface, 0, 0);
   cairo_rectangle(cairo, 0, FEEDBACK_AREA_Y, board_width(), feedback_area_height);
   cairo_fill(cairo);
   
   g_idle_add(update_drawing_area, NULL);
}


void
draw_message (const gchar *str1, const gchar *str2, gboolean center)
{
   gchar       *fstr;
   PangoLayout *layout;
   PangoFontDescription *desc;
   gint width, height;
   gint x, y;
   
   g_assert(str1 != NULL);
   
   clear_message_area();
   
   setup_text_gc();
   
   fstr = ini_get_string(FONTS, TEXT_FONT, DEF_TEXT_FONT);
   desc = pango_font_description_from_string(fstr);
   g_free (fstr);
   
   layout = pango_cairo_create_layout(cairo);
   pango_layout_set_font_description(layout, desc);
   pango_layout_set_text(layout, str1, -1);
   pango_layout_get_pixel_size(layout, &width, &height);
   x = TEXT_X_PADDING;
   if (center) {
	  x = (board_width() - width) / 2;
      x = (x < TEXT_X_PADDING) ? TEXT_X_PADDING : x;
   }
   y = FEEDBACK_AREA_Y + TEXT_Y_PADDING;
   cairo_translate(cairo, x, y);
   pango_cairo_update_layout(cairo, layout);
   pango_cairo_show_layout(cairo, layout);
   cairo_translate(cairo, -x, -y);
   g_object_unref(layout);
   
   if (str2 != NULL) {
	  layout = pango_cairo_create_layout(cairo);
      pango_layout_set_font_description(layout, desc);
      pango_layout_set_text(layout, str2, -1);
      pango_layout_get_pixel_size(layout, &width, &height);
      if (center) {
	      x = (board_width() - width) / 2;
          x = (x < TEXT_X_PADDING) ? TEXT_X_PADDING : x;
      }
      y += (TEXT_Y_PADDING + height);
      cairo_translate(cairo, x, y);
      pango_cairo_update_layout(cairo, layout);
      pango_cairo_show_layout(cairo, layout);
      cairo_translate(cairo, -x, -y);
      g_object_unref(layout); 
   }
   pango_font_description_free(desc);
   
   g_idle_add(update_drawing_area, NULL);
}


void
refresh_board (const player_object *player1, const player_object *player2)
{
   clear_message_area();
	
   setup_text_gc();

   draw_scoreboard(player1, player2);
}


void
setup_text_gc (void)
{
   gchar *colorstr;
   GdkRGBA rgba;
   
   colorstr = ini_get_string(FONTS, FONT_COLOR, DEF_COLOR);
   if (!gdk_rgba_parse(&rgba, (const gchar *)colorstr)) {
	  gdk_rgba_parse(&rgba, "white");
   }
   g_free(colorstr);
   cairo_set_source_rgb(cairo, rgba.red, rgba.green, rgba.blue);
   
   cairo_set_line_width(cairo, LINE_WIDTH);
   cairo_set_line_cap(cairo, CAIRO_LINE_CAP_ROUND);
   cairo_set_line_join(cairo, CAIRO_LINE_JOIN_ROUND);
}

cairo_surface_t *
get_draw_surface(void)
{
	return surface;
}

void
init_draw_objects (void)
{
   gchar *fstr, *tmpstr;
   PangoLayout *layout;
   PangoFontDescription *desc;
   gint width, height;
   cairo_surface_t *tmpsurface;
   cairo_t *tmpcairo;
   cairo_pattern_t *tmppattern;
   
   g_return_if_fail(surface == NULL);
   g_return_if_fail(cairo == NULL);
      
   /*
    * Check the size of the normal display font.
    *
    * I use "A4gp" to estimate the height that will be used as it 
    * should provide a maximal height since it has an upper case
    * letter, a number, and two lower case numbers that dip
    * below the baseline.
    */
   
   tmpsurface = cairo_image_surface_create(CAIRO_FORMAT_RGB24, board_width(), board_height());
   tmpcairo = cairo_create(tmpsurface);
   
   fstr = ini_get_string(FONTS, TEXT_FONT, DEF_TEXT_FONT);
   desc = pango_font_description_from_string(fstr);
   g_free (fstr);
   
   layout = pango_cairo_create_layout(tmpcairo);
   pango_layout_set_font_description(layout, desc);
   pango_font_description_free(desc);
   pango_layout_set_text(layout, "A4gp", -1);
   pango_layout_get_pixel_size(layout, &width, &height);
   g_object_unref(layout);
   
   score_area_height = (4 * TEXT_Y_PADDING) + (3 * height);
   feedback_area_height = (3 * TEXT_Y_PADDING) + (2 * height);
   
   cairo_destroy(tmpcairo);
   cairo_surface_destroy(tmpsurface);
   
   /*
    * Setup the main GC that is used when drawing in the drawing area.
    */
   tmpstr = g_build_filename(PACKAGE_DATA_DIR, "pixmaps", "bg.png", NULL);
   tmpsurface = cairo_image_surface_create_from_png(tmpstr);
   tmppattern = cairo_pattern_create_for_surface(tmpsurface);
   g_free(tmpstr);
   
   surface = cairo_image_surface_create(CAIRO_FORMAT_RGB24, board_width(), board_height());
   cairo = cairo_create(surface);
   cairo_set_source(cairo, tmppattern);
   cairo_pattern_set_extend(cairo_get_source(cairo), CAIRO_EXTEND_REPEAT);
   cairo_rectangle(cairo, 0, 0, board_width(), board_height());
   cairo_fill(cairo);
   
   bgsurface = cairo_image_surface_create(CAIRO_FORMAT_RGB24, board_width(), board_height());
   bgcairo = cairo_create(bgsurface);
   cairo_set_source(bgcairo, tmppattern);
   cairo_pattern_set_extend(cairo_get_source(bgcairo), CAIRO_EXTEND_REPEAT);
   cairo_rectangle(bgcairo, 0, 0, board_width(), board_height());
   cairo_fill(bgcairo);
   
   cairo_pattern_destroy(tmppattern);
   cairo_surface_destroy(tmpsurface);
   
   /*
    * Setup the GC used to draw the operators
    */
   setup_text_gc();
}

void
destroy_draw_objects (void)
{
	cairo_surface_destroy(bgsurface);
	cairo_destroy(bgcairo);
	
	cairo_surface_destroy(surface);
	cairo_destroy(cairo);
}


void
draw_pause (void)
{
   gchar       *fstr;
   PangoLayout *layout;
   PangoFontDescription *desc;
   gint width, height;
   gint x, y;
   
   cairo_set_source_surface(cairo, bgsurface, 0, 0);
   cairo_rectangle(cairo, 0, 0, board_width(), board_height());
   cairo_fill(cairo);
   
   setup_text_gc();
   
   fstr = ini_get_string(FONTS, TITLE_FONT, DEF_TITLE_FONT);
   desc = pango_font_description_from_string(fstr);
   g_free (fstr);
   
   layout = pango_cairo_create_layout(cairo);
   pango_layout_set_font_description(layout, desc);
   pango_font_description_free(desc);
   pango_layout_set_text(layout, _("Pause"), -1);
   pango_layout_get_pixel_size(layout, &width, &height);
   x = board_width() / 2 - (width / 2);
   y = board_height() / 2 - (height / 2);
   cairo_translate(cairo, x, y);
   pango_cairo_update_layout(cairo, layout);
   pango_cairo_show_layout(cairo, layout);
   cairo_translate(cairo, -x, -y);
   g_object_unref(layout);
   
   g_idle_add(update_drawing_area, NULL);
}

void
draw_game_over (const game_engine *game)
{
   gchar *fstr, *tmpstr;
   PangoFontDescription *fontdesc, *titledesc;
   PangoLayout *tplayout, *aglayout, *cglayout, *layout;
   gint width, height;
   guint tpheight, agheight, cgheight;
   gint x, y, statx;
   
   cairo_set_source_surface(cairo, bgsurface, 0, 0);
   cairo_rectangle(cairo, 0, 0, board_width(), board_height());
   cairo_fill(cairo);
   
   setup_text_gc();
   
   fstr = ini_get_string(FONTS, TEXT_FONT, DEF_TEXT_FONT);
   fontdesc = pango_font_description_from_string(fstr);
   g_free (fstr);
   
   fstr = ini_get_string(FONTS, TITLE_FONT, DEF_TITLE_FONT);
   titledesc = pango_font_description_from_string(fstr);
   g_free(fstr);
   
   /* 
    * The score text will be lined up after the longest of the 
    * "prompt" strings.  Determine which one is longest, and how
    * long it is.
    */
   tplayout = pango_cairo_create_layout(cairo);
   pango_layout_set_font_description(tplayout, fontdesc);
   pango_layout_set_text(tplayout, _("Total Points:"), -1);
   pango_layout_get_pixel_size(tplayout, &width, &height); 
   tpheight = height;
   statx = width;
   
   aglayout = pango_cairo_create_layout(cairo);
   pango_layout_set_font_description(aglayout, fontdesc);
   pango_layout_set_text(aglayout, _("Guesses:"), -1);
   pango_layout_get_pixel_size(aglayout, &width, &height); 
   agheight = height;
   statx = (width > statx) ? width : statx;
   
   cglayout = pango_cairo_create_layout(cairo);
   pango_layout_set_font_description(cglayout, fontdesc);
   pango_layout_set_text(cglayout, _("Correct Guesses:"), -1);
   pango_layout_get_pixel_size(cglayout, &width, &height);
   cgheight = height;
   statx = (width > statx) ? width : statx;
   statx += (TEXT_INDENT + TEXT_X_PADDING);
   
   /*
    * Game Over text...
    */
   layout = pango_cairo_create_layout(cairo);
   pango_layout_set_font_description(layout, titledesc);
   pango_layout_set_text(layout, _("Game Over!!"), -1);
   pango_layout_get_pixel_size(layout, &width, &height);
   x = board_width() / 2 - (width / 2);
   y = 0;
   cairo_translate(cairo, x, y);
   pango_cairo_update_layout(cairo, layout);
   pango_cairo_show_layout(cairo, layout);
   cairo_translate(cairo, -x, -y);
   g_object_unref(layout);
   
   /*
    * Statisticts
    */
   layout = pango_cairo_create_layout(cairo);
   pango_layout_set_font_description(layout, titledesc);
   pango_layout_set_text(layout, _("Scores:"), -1);
   pango_layout_get_pixel_size(layout, &width, &height);
   x = board_width() / 2 - (width / 2);
   y = y + height;
   cairo_translate(cairo, x, y);
   pango_cairo_update_layout(cairo, layout);
   pango_cairo_show_layout(cairo, layout);
   cairo_translate(cairo, -x, -y);
   g_object_unref(layout); 
      
   /*
    * Player 1
    */
   tmpstr = g_strdup_printf("%s:", get_player_name(game->player1));
   layout = pango_cairo_create_layout(cairo);
   pango_layout_set_font_description(layout, fontdesc);
   pango_layout_set_text(layout, tmpstr, -1);
   pango_layout_get_pixel_size(layout, &width, &height);
   x = TEXT_X_PADDING;
   y += height * 2;
   cairo_translate(cairo, x, y);
   pango_cairo_update_layout(cairo, layout);
   pango_cairo_show_layout(cairo, layout);
   cairo_translate(cairo, -x, -y);
   g_object_unref(layout); 
   g_free(tmpstr);
   
   x = TEXT_INDENT;
   y += tpheight + TEXT_Y_PADDING;
   cairo_translate(cairo, x, y);
   pango_cairo_update_layout(cairo, tplayout);
   pango_cairo_show_layout(cairo, tplayout);
   cairo_translate(cairo, -x, -y);
   
   tmpstr = g_strdup_printf("%d", get_player_score(game->player1));
   layout = pango_cairo_create_layout(cairo);
   pango_layout_set_font_description(layout, fontdesc);
   pango_layout_set_text(layout, tmpstr, -1);
   cairo_translate(cairo, statx, y);
   pango_cairo_update_layout(cairo, layout);
   pango_cairo_show_layout(cairo, layout);
   cairo_translate(cairo, -statx, -y);
   g_object_unref(layout); 
   g_free(tmpstr);
   
   x = TEXT_INDENT;
   y += agheight + TEXT_Y_PADDING;
   cairo_translate(cairo, x, y);
   pango_cairo_update_layout(cairo, aglayout);
   pango_cairo_show_layout(cairo, aglayout);
   cairo_translate(cairo, -x, -y);
   
   tmpstr = g_strdup_printf("%d", get_player_guesses(game->player1));
   layout = pango_cairo_create_layout(cairo);
   pango_layout_set_font_description(layout, fontdesc);
   pango_layout_set_text(layout, tmpstr, -1);
   cairo_translate(cairo, statx, y);
   pango_cairo_update_layout(cairo, layout);
   pango_cairo_show_layout(cairo, layout);
   cairo_translate(cairo, -statx, -y);
   g_object_unref(layout); 
   g_free(tmpstr);
   
   x = TEXT_INDENT;
   y += cgheight + TEXT_Y_PADDING;
   cairo_translate(cairo, x, y);
   pango_cairo_update_layout(cairo, cglayout);
   pango_cairo_show_layout(cairo, cglayout);
   cairo_translate(cairo, -x, -y);
   
   tmpstr = g_strdup_printf("%d", get_player_correct_guesses(game->player1));
   layout = pango_cairo_create_layout(cairo);
   pango_layout_set_font_description(layout, fontdesc);
   pango_layout_set_text(layout, tmpstr, -1);
   cairo_translate(cairo, statx, y);
   pango_cairo_update_layout(cairo, layout);
   pango_cairo_show_layout(cairo, layout);
   cairo_translate(cairo, -statx, -y);
   g_object_unref(layout); 
   g_free(tmpstr);
   
   /*
    * Player 2
    */
   tmpstr = g_strdup_printf("%s:", get_player_name(game->player2));
   layout = pango_cairo_create_layout(cairo);
   pango_layout_set_font_description(layout, fontdesc);
   pango_layout_set_text(layout, tmpstr, -1);
   pango_layout_get_pixel_size(layout, &width, &height);
   x = TEXT_X_PADDING;
   y += height * 2;
   cairo_translate(cairo, x, y);
   pango_cairo_update_layout(cairo, layout);
   pango_cairo_show_layout(cairo, layout);
   cairo_translate(cairo, -x, -y);
   g_object_unref(layout); 
   g_free(tmpstr);
   
   x = TEXT_INDENT;
   y += tpheight + TEXT_Y_PADDING;
   cairo_translate(cairo, x, y);
   pango_cairo_update_layout(cairo, tplayout);
   pango_cairo_show_layout(cairo, tplayout);
   cairo_translate(cairo, -x, -y);
   
   tmpstr = g_strdup_printf("%d", get_player_score(game->player2));
   layout = pango_cairo_create_layout(cairo);
   pango_layout_set_font_description(layout, fontdesc);
   pango_layout_set_text(layout, tmpstr, -1);
   cairo_translate(cairo, statx, y);
   pango_cairo_update_layout(cairo, layout);
   pango_cairo_show_layout(cairo, layout);
   cairo_translate(cairo, -statx, -y);
   g_object_unref(layout); 
   g_free(tmpstr);
   
   x = TEXT_INDENT;
   y += agheight + TEXT_Y_PADDING;
   cairo_translate(cairo, x, y);
   pango_cairo_update_layout(cairo, aglayout);
   pango_cairo_show_layout(cairo, aglayout);
   cairo_translate(cairo, -x, -y);
   
   tmpstr = g_strdup_printf("%d", get_player_guesses(game->player2));
   layout = pango_cairo_create_layout(cairo);
   pango_layout_set_font_description(layout, fontdesc);
   pango_layout_set_text(layout, tmpstr, -1);
   cairo_translate(cairo, statx, y);
   pango_cairo_update_layout(cairo, layout);
   pango_cairo_show_layout(cairo, layout);
   cairo_translate(cairo, -statx, -y);
   g_object_unref(layout); 
   g_free(tmpstr);
   
   x = TEXT_INDENT;
   y += cgheight + TEXT_Y_PADDING;
   cairo_translate(cairo, x, y);
   pango_cairo_update_layout(cairo, cglayout);
   pango_cairo_show_layout(cairo, cglayout);
   cairo_translate(cairo, -x, -y);
   
   tmpstr = g_strdup_printf("%d", get_player_correct_guesses(game->player2));
   layout = pango_cairo_create_layout(cairo);
   pango_layout_set_font_description(layout, fontdesc);
   pango_layout_set_text(layout, tmpstr, -1);
   cairo_translate(cairo, statx, y);
   pango_cairo_update_layout(cairo, layout);
   pango_cairo_show_layout(cairo, layout);
   cairo_translate(cairo, -statx, -y);
   g_object_unref(layout); 
   g_free(tmpstr);
   
   g_object_unref(tplayout);
   g_object_unref(cglayout);
   g_object_unref(aglayout);
   pango_font_description_free(titledesc);
   pango_font_description_free(fontdesc);
   
   g_idle_add(update_drawing_area, NULL);
}
