/*-------------------------------------------------------------------------
 * Copyright (c) 2000-2007 Kenneth W. Sodemann (ken.sodemann@gmail.com)
 * Copyright (c) 2014, Alex Panin (alex@linuxonly.ru) and Alexey Loginov (loginov.alex.valer@gmail.com)
 *-------------------------------------------------------------------------
 * ini_defs
 *
 * Synopsis:
 *   Defines used with the INI files
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to
 * Free Software Foundation, Inc.
 * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 *-------------------------------------------------------------------------
 */
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <glib/gi18n.h>
#include <glib/gstdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>


#include "ini_defs.h"

#define  BASIC_CONF         "/%s/%s/%s"
#define  CONF_WITH_INT_DEF  "/%s/%s/%s=%d"
#define  CONF_WITH_STR_DEF  "/%s/%s/%s=%s"

static GKeyFile *keyfile  = NULL;
static gchar *keyfilepath = NULL;

gboolean
ini_load_settings(void)
{
	GError *error;
	
	keyfilepath = g_build_filename(g_get_user_config_dir(), "mathwar", "settings.conf", NULL);
	
	keyfile = g_key_file_new();
	
	error = NULL;
	
	if (g_file_test((const gchar *)keyfilepath, G_FILE_TEST_IS_REGULAR)) {
		if (!g_key_file_load_from_file(keyfile, (const gchar *)keyfilepath, G_KEY_FILE_NONE, &error)) {
			g_warning("Unable to read settings from file: %s", error->message);
			g_error_free(error);
			return FALSE;
		}
	}
	
	return TRUE;
}

gboolean
ini_save_settings(void)
{
	gchar *configpath;
	gchar *data;
	gsize length;
	GError *error;
	gint fd;
	
	if ((keyfile != NULL) && (keyfilepath != NULL)) {
		configpath = g_build_path(G_DIR_SEPARATOR_S, g_get_user_config_dir(), "mathwar", NULL);
		if (!g_file_test((const gchar *)configpath, G_FILE_TEST_IS_DIR)) {
			if (g_mkdir_with_parents((const gchar *)configpath, 0755) == -1) {
				g_warning("Unable to create directory for configuration file");
				g_free(configpath);
				return FALSE;
			}
		}
		g_free(configpath);
		
		if (!g_file_test((const gchar *)keyfilepath, G_FILE_TEST_IS_REGULAR)) {
			fd = g_creat((const gchar *)keyfilepath, 0755);
			if (fd == -1) {
				g_warning("Unable to create configuration file");
				return FALSE;
			}
			close(fd);
		}
		
		error = NULL;
		data = g_key_file_to_data(keyfile, &length, &error);
		if (data != NULL) {
			error = NULL;
			if (g_file_set_contents((const gchar *)keyfilepath, (const gchar *)data, (gssize)length, &error)) {
				g_free(data);
				return TRUE;
			} else {
				g_warning("Unable to write keyfile data to file: %s", error->message);
				g_error_free(error);
				g_free(data);
			}
			
		} else {
			g_warning("Unable to get keyfile data: %s", error->message);
			g_error_free(error);
		}
	}
	
	return FALSE;
}

void
ini_close(void)
{
	if (keyfile != NULL) {
		g_key_file_unref(keyfile);
		keyfile = NULL;
	}
	
	if	(keyfilepath != NULL) {
		g_free(keyfilepath);
		keyfilepath = NULL;
	}
}

gboolean
ini_forget_changes(void)
{
	GError *error;
	
	if ((keyfile != NULL) && (keyfilepath != NULL)) {
		if (g_file_test((const gchar *)keyfilepath, G_FILE_TEST_IS_REGULAR)) {
			if (!g_key_file_load_from_file(keyfile, (const gchar *)keyfilepath, G_KEY_FILE_NONE, &error)) {
				g_warning("Unable to read settings from file: %s", error->message);
				g_error_free(error);
				return FALSE;
			}
		}
	}
	
	return TRUE;
}

gint
ini_get_int (const gchar *section,
             const gchar *label,
             gint         def_val)
{
   gint rtn_val;
   GError *error;
	
   if (keyfile == NULL) return def_val;
   
   error = NULL;
   
   rtn_val = g_key_file_get_integer(keyfile, section, label, &error);
   
   if (error != NULL) {
	   g_debug("Unable to get integer setting %s", error->message);
	   g_error_free(error);
	   return def_val;
   }
   
   return rtn_val;
}

void
ini_set_int (const gchar *section,
             const gchar *label,
             gint         value)
{
   if (keyfile == NULL) return;
   
   g_key_file_set_integer(keyfile, section, label, value);
}


gchar *
ini_get_string (const gchar *section,
                const gchar *label,
                const gchar *def_val)
{
   gchar *rtn;
   GError *error;
	
   if (keyfile == NULL) return g_strdup(def_val);
   
   error = NULL;
   
   rtn = g_key_file_get_string(keyfile, section, label, &error);
   
   if (error != NULL) {
	   g_debug("Unable to get string setting %s", error->message);
	   g_error_free(error);
	   return g_strdup(def_val);
   }
   
   return rtn;
}

void
ini_set_string (const gchar *section,
                const gchar *label,
                const gchar *value)
{
   if ((keyfile == NULL) || (section == NULL) || (label == NULL) || (value == NULL)) return;
   
   g_key_file_set_string(keyfile, section, label, value);
}
