/*-------------------------------------------------------------------------
 * Copyright (c) 2000-2007 Kenneth W. Sodemann (ken.sodemann@gmail.com)
 * Copyright (c) 2014, Alex Panin (alex@linuxonly.ru) and Alexey Loginov (loginov.alex.valer@gmail.com)
 *-------------------------------------------------------------------------
 * mainwin
 *
 * Synopsis:
 *   The main MathWars window
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to
 * Free Software Foundation, Inc.
 * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 *-------------------------------------------------------------------------
 */
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <glib/gi18n.h>
#include <ctype.h>

#include "about_dlg.h"
#include "draw.h"
#include "game_engine.h"
#include "mainwin.h"
#include "operators.h"
#include "prop_dlg.h"

#define MAINWIN        "main_window"
#define DRAWING_GC     "drawing_gc"
#define DRAWING_AREA   "drawing_area"
#define MAIN_APPBAR    "appbar1"
#define TIMER_DIAL     "timer_dial"
#define GAME_ENGINE    "game_eng"
#define ANSWER_ENTRY   "answer_entry"
#define SUBMIT_BUTTON  "submit_button"
#define PAUSE_BUTTON   "pause_button"
#define YES_BUTTON     "yes_button"
#define NO_BUTTON      "no_button"
#define NEW_GAME_MENU  "new_game_menu"

static GtkWidget *mainwin;
static GtkWidget *submit_button;
static GtkWidget *no_button;
static GtkWidget *yes_button;
static GtkWidget *answer_entry;
static GtkWidget *pause_button;
static GtkWidget *drawingarea;
static GtkWidget *statusbar;
static gulong limit_input_handler;


static gboolean draw_event(GtkWidget *widget, cairo_t *cr)
{
   cairo_surface_t *drawsurface;
   
   drawsurface = get_draw_surface();
   
   if (drawsurface != NULL) {
	   cairo_set_source_surface(cr, drawsurface, 0, 0);
	   cairo_rectangle(cr, 0, 0, board_width(), board_height());
	   cairo_fill(cr);
   }
   
   return TRUE;
}

gboolean update_drawing_area(gpointer data)
{
	gtk_widget_queue_draw(drawingarea);
	
	return FALSE;
}

static void
new_game_menu_item_activate_signal(GSimpleAction *action, GVariant *parameter, gpointer data)
{
   game_engine *game;
   
   g_assert(mainwin);
   
   game = g_object_get_data(G_OBJECT(mainwin), GAME_ENGINE);
   g_assert(game);
   
   begin_game(game);
   begin_round(game);
}

static void
preferences_menu_item_activate_signal(GSimpleAction *action, GVariant *parameter, gpointer data)
{
   g_assert(mainwin);
   show_prop_dlg(GTK_WIDGET(mainwin));
}

static void
about_menu_item_activate_signal(GSimpleAction *action, GVariant *parameter, gpointer data)
{
   g_assert (mainwin != NULL);
   show_about_dlg(mainwin);
}

static void
exit_menu_item_activate_signal(GSimpleAction *action, GVariant *parameter, gpointer data)
{
   GtkApplication *application;
   game_engine *game;
   
   application = (GtkApplication *)data;
   
   g_assert(application);
   g_assert(mainwin);
   
   game = g_object_get_data(G_OBJECT(mainwin), GAME_ENGINE);
   
   g_assert(game);
   
   destroy_game_engine(game);
   
   g_application_quit(G_APPLICATION(application));
}

static void
on_mainwin_show (GtkWidget     *widget,
                 gpointer       user_data)
{
   game_engine *game;

   game = create_game_engine (widget);
   g_object_set_data(G_OBJECT(widget), GAME_ENGINE, game);

   /*
    * init_draw_objects() will initialize certain objects that
    * determine the size of the drawing area.  Thus, set the
    * size here instead of in the create function.
    */
   init_draw_objects();
   gtk_widget_set_size_request(GTK_WIDGET(drawingarea), board_width(), board_height());
   draw_cards_down();

   refresh_board(get_player1(game), get_player2(game));
}


static void
on_mainwin_destroy(GtkWidget  *widget,
                   gpointer    data)
{
   GtkApplication *application;
   game_engine *game;
   
   application = (GtkApplication *)data;
   
   g_assert(application);
   g_assert(mainwin);

   game = g_object_get_data(G_OBJECT(widget), GAME_ENGINE);
   
   g_assert(game);
   
   destroy_game_engine(game);
   
   g_application_quit(G_APPLICATION(application));
}


static void
limit_input_to_nums (GtkEntry    *entry,
                     const gchar *text,
                     gint         length,
                     gint        *position,
                     gpointer     data)
{
   GtkEditable *edit = GTK_EDITABLE (entry);
   gint         idx;
   gint         cnt = 0;
   gchar       *res = g_new (gchar, length);

   for (idx = 0; idx < length; idx++) {
      if (text[idx] == '-' || (isdigit (text[idx]))) {
         res[cnt++] = text[idx];
      }
   }

   if (cnt > 0) {
      g_signal_handler_block(G_OBJECT(edit), limit_input_handler); 
      gtk_editable_insert_text(edit, res, cnt, position);
      g_signal_handler_unblock(G_OBJECT(edit), limit_input_handler);
   }
   g_signal_stop_emission_by_name(G_OBJECT(edit), "insert_text");

   g_free (res);
}


static gboolean
on_answer_entry_key_press_event (GtkWidget       *widget,
                                 GdkEventKey     *event,
                                 gpointer         user_data)
{
   if (event->keyval == GDK_KEY_Return || event->keyval == GDK_KEY_KP_Enter) {
	  g_assert(submit_button);
	  g_signal_emit_by_name(G_OBJECT(submit_button), "clicked");
   }

   return FALSE;
}


static void
on_yes_clicked (GtkWidget  *widget,
                gpointer    user_data)
{
   game_engine *game;

   /*
    * Right now, the only time this button is active is if the 
    * computer player has made a guess, and the user is answering
    * the bonus question.  The game engine will handle most of the
    * actual logic.  Just let the game engine know that the user
    * pressed the button, and then disable both the yes button and
    * the no button.
    */
   g_assert(mainwin != NULL);
   g_assert(no_button != NULL);
   
   game = g_object_get_data(G_OBJECT(mainwin), GAME_ENGINE);
   g_assert (game != NULL);
   
   gtk_widget_set_sensitive(no_button, FALSE);
   gtk_widget_set_sensitive(widget, FALSE);
   handle_yes_answer(game);
}


static void
on_no_clicked (GtkWidget  *widget,
               gpointer    user_data)
{
   game_engine *game;

   /*
    * Right now, the only time this button is active is if the 
    * computer player has made a guess, and the user is answering
    * the bonus question.  The game engine will handle most of the
    * actual logic.  Just let the game engine know that the user
    * pressed the button, and then disable both the yes button and
    * the no button.
    */
   g_assert(mainwin != NULL);
   g_assert(yes_button != NULL);
   
   game = g_object_get_data(G_OBJECT(mainwin), GAME_ENGINE);
   g_assert (game != NULL);

   gtk_widget_set_sensitive(yes_button, FALSE);
   gtk_widget_set_sensitive(widget, FALSE);
   handle_no_answer(game);
}


static void
on_submit_clicked (GtkWidget  *widget,
                   gpointer    user_data)
{
   game_engine *game;
      
   g_assert(mainwin);
   g_assert (answer_entry);
   g_assert (pause_button);
   
   game = g_object_get_data(G_OBJECT (mainwin), GAME_ENGINE);
   g_assert(game);

   gtk_widget_set_sensitive(widget, FALSE);

   handle_player_answer(game, gtk_entry_get_text(GTK_ENTRY(answer_entry)));
}


static void
on_pause_toggled (GtkWidget *pause_button,
                  gpointer   data)
{
   game_engine  *game;
   GtkApplication *application;
   GAction *action;
   static gboolean submit_active = TRUE;
   static gboolean yes_active    = FALSE;
   static gboolean no_active     = FALSE;
   
   
   application = (GtkApplication *)data;
   
   g_assert(application);
   
   action = g_action_map_lookup_action(G_ACTION_MAP(application), "newgame");
   
   g_assert(action);
   
   /*
    * static vars used to store the current state of the buttons.
    * The current state should be saved before pausing, and 
    * restored when the game is no longer paused.
    *
    * There should not be any event that occurs during a pause
    * that will validly change the states.
    */
   
   g_assert (mainwin);
   
   game = g_object_get_data(G_OBJECT(mainwin), GAME_ENGINE);
   g_assert(game);

   if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(pause_button))) {
      /*
       * Save the state of the submit, yes, and no buttons, then
       * disable them.
       */
      g_assert(submit_button != NULL);
      submit_active = gtk_widget_get_sensitive(submit_button);
      gtk_widget_set_sensitive(submit_button, FALSE);
      
      g_assert(yes_button != NULL);
      yes_active = gtk_widget_get_sensitive(yes_button);
      gtk_widget_set_sensitive(yes_button, FALSE);
      
      g_assert(no_button != NULL);
      no_active = gtk_widget_get_sensitive(no_button);
      gtk_widget_set_sensitive(no_button, FALSE);

      /*
       * Disable the "New Game" menu item.
       */
      g_simple_action_set_enabled(G_SIMPLE_ACTION(action), FALSE);

      pause_game (game);
   } else {
      /*
       * Restore the submit, yes, and no buttons before resuming the
       * game.
       */
      g_assert(submit_button != NULL);
      gtk_widget_set_sensitive(submit_button, submit_active);
      
      g_assert(yes_button != NULL);
      gtk_widget_set_sensitive(yes_button, yes_active);
      
      g_assert(no_button != NULL);
      gtk_widget_set_sensitive(no_button, no_active);

      /*
       * Enable the "New Game" menu item.
       */
      g_simple_action_set_enabled(G_SIMPLE_ACTION(action), TRUE);

      restore_game (game);
   }
}

GtkWidget *
create_mainwin(GtkApplication *application)
{
  GtkWidget   *vbox1;
  GtkWidget   *hbox1;
  GtkWidget   *hbox2;
  GtkWidget   *hsep;
  GtkWidget   *button_image;
  GMenu *menu, *actsection, *appsection, *prefsection, *helpsection, *quitsection;
  static GActionEntry app_actions[] = {
    { "newgame", new_game_menu_item_activate_signal, NULL, NULL, NULL },
    { "preferences", preferences_menu_item_activate_signal, NULL, NULL, NULL },
    { "about", about_menu_item_activate_signal, NULL, NULL, NULL },
    { "quit", exit_menu_item_activate_signal, NULL, NULL, NULL },
  };

  mainwin = gtk_application_window_new(application);
  gtk_application_window_set_show_menubar(GTK_APPLICATION_WINDOW(mainwin), TRUE);
  gtk_window_set_title(GTK_WINDOW(mainwin), _(PACKAGE));
  gtk_window_set_resizable(GTK_WINDOW(mainwin), FALSE);
  g_signal_connect(G_OBJECT(mainwin), "show", G_CALLBACK(on_mainwin_show), NULL);
  g_signal_connect(G_OBJECT(mainwin), "destroy", G_CALLBACK(on_mainwin_destroy), application);
  
  g_action_map_add_action_entries(G_ACTION_MAP(application), app_actions, G_N_ELEMENTS(app_actions), application);
  
  menu = g_menu_new();
  
  appsection = g_menu_new();
  g_menu_append(appsection, _(N_("_New Game")), "app.newgame");
  #if GTK_CHECK_VERSION(3,12,0)
    gtk_application_set_accels_for_action(application, "app.newgame", (const gchar*[]) {"<Primary>N", NULL});
  #else
	gtk_application_add_accelerator(application, "<Primary>N", "app.newgame", NULL);
  #endif
  quitsection = g_menu_new();
  g_menu_append(quitsection, _("_Quit"), "app.quit");
  #if GTK_CHECK_VERSION(3,12,0)
    gtk_application_set_accels_for_action(application, "app.quit", (const gchar*[]) {"<Primary>q", NULL});
  #else
	gtk_application_add_accelerator(application, "<Primary>q", "app.quit", NULL);
  #endif
  actsection = g_menu_new();
  g_menu_append_section(actsection, NULL, G_MENU_MODEL(appsection));
  g_menu_append_section(actsection, NULL, G_MENU_MODEL(quitsection));
  g_menu_append_submenu(menu, _("_Actions"), G_MENU_MODEL(actsection));
  
  prefsection = g_menu_new();
  g_menu_append(prefsection, _("_Preferences"), "app.preferences");
  #if GTK_CHECK_VERSION(3,12,0)
    gtk_application_set_accels_for_action(application, "app.preferences", (const gchar*[]) {"<Primary>p", NULL});
  #else
    gtk_application_add_accelerator(application, "<Primary>p", "app.preferences", NULL);
  #endif
  g_menu_append_submenu(menu, _("_Edit"), G_MENU_MODEL(prefsection));
  
  helpsection = g_menu_new();
  g_menu_append(helpsection, _("_About"), "app.about");
  g_menu_append_submenu(menu, _("_Help"), G_MENU_MODEL(helpsection));
  
  gtk_application_set_menubar(application, G_MENU_MODEL(menu));
  g_object_unref(menu);
  
  vbox1 = gtk_box_new(GTK_ORIENTATION_VERTICAL, 5);
  gtk_container_set_border_width(GTK_CONTAINER(vbox1), 0);
  gtk_container_add(GTK_CONTAINER(mainwin), vbox1);

  drawingarea = gtk_drawing_area_new();
  gtk_box_pack_start(GTK_BOX (vbox1), drawingarea, TRUE, TRUE, 0);
  gtk_widget_set_size_request(GTK_WIDGET(drawingarea), board_width(), board_height());
  g_signal_connect(drawingarea, "draw", G_CALLBACK(draw_event), NULL);
  
  hsep = gtk_separator_new(GTK_ORIENTATION_HORIZONTAL);
  gtk_box_pack_start(GTK_BOX(vbox1), hsep, FALSE, FALSE, 0);

  hbox1 = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 10);
  gtk_box_pack_start(GTK_BOX(vbox1), hbox1, FALSE, FALSE, 0);
  gtk_container_set_border_width(GTK_CONTAINER(hbox1), 5);

  answer_entry = gtk_entry_new();
  gtk_box_pack_start(GTK_BOX(hbox1), answer_entry, TRUE, TRUE, 0);
  limit_input_handler = g_signal_connect(G_OBJECT(answer_entry), "insert_text", G_CALLBACK(limit_input_to_nums), NULL);
  g_signal_connect(G_OBJECT(answer_entry), "key_press_event", G_CALLBACK(on_answer_entry_key_press_event), NULL);

  hbox2 = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
  gtk_box_pack_start(GTK_BOX(hbox1), hbox2, FALSE, TRUE, 0);
  submit_button = gtk_button_new_with_label(_("Submit"));
  #if GTK_CHECK_VERSION(3,10,0)
    button_image = gtk_image_new_from_icon_name("media-seek-forward", GTK_ICON_SIZE_BUTTON);
  #else
	button_image = gtk_image_new_from_stock(GTK_STOCK_MEDIA_FORWARD, GTK_ICON_SIZE_BUTTON);
  #endif
  gtk_button_set_image(GTK_BUTTON(submit_button), button_image);
  gtk_box_pack_start(GTK_BOX(hbox2), submit_button, FALSE, TRUE, 0);
  gtk_widget_set_sensitive(submit_button, FALSE);
  gtk_widget_set_can_default(submit_button, TRUE);
  g_signal_connect(G_OBJECT(submit_button), "clicked", G_CALLBACK(on_submit_clicked), NULL);
  
  pause_button = gtk_toggle_button_new_with_label(_("Pause"));
  #if GTK_CHECK_VERSION(3,10,0)
    button_image = gtk_image_new_from_icon_name("media-playback-pause", GTK_ICON_SIZE_BUTTON);
  #else
	button_image = gtk_image_new_from_stock(GTK_STOCK_MEDIA_PAUSE, GTK_ICON_SIZE_BUTTON);
  #endif
  gtk_button_set_image(GTK_BUTTON(pause_button), button_image);
  gtk_box_pack_start(GTK_BOX(hbox2), pause_button, FALSE, TRUE, 0);
  gtk_widget_set_sensitive (pause_button, FALSE);
  gtk_widget_set_can_default(pause_button, TRUE);
  g_signal_connect(G_OBJECT(pause_button), "toggled", G_CALLBACK(on_pause_toggled), application);

  yes_button = gtk_button_new_with_label(_("Yes"));
  #if GTK_CHECK_VERSION(3,10,0)
    button_image = gtk_image_new_from_icon_name("dialog-ok", GTK_ICON_SIZE_BUTTON);
  #else
	button_image = gtk_image_new_from_stock(GTK_STOCK_YES, GTK_ICON_SIZE_BUTTON);
  #endif
  gtk_button_set_image(GTK_BUTTON(yes_button), button_image);
  gtk_box_pack_start(GTK_BOX(hbox2), yes_button, FALSE, TRUE, 0);
  gtk_widget_set_sensitive(yes_button, FALSE);
  gtk_widget_set_can_default(yes_button, TRUE);
  g_signal_connect(G_OBJECT(yes_button), "clicked", G_CALLBACK(on_yes_clicked), NULL);

  no_button = gtk_button_new_with_label(_("No"));
  #if GTK_CHECK_VERSION(3,10,0)
    button_image = gtk_image_new_from_icon_name("dialog-cancel", GTK_ICON_SIZE_BUTTON);
  #else
	button_image = gtk_image_new_from_stock(GTK_STOCK_NO, GTK_ICON_SIZE_BUTTON);
  #endif
  gtk_button_set_image(GTK_BUTTON(no_button), button_image);
  gtk_box_pack_start(GTK_BOX(hbox2), no_button, FALSE, TRUE, 0);
  gtk_widget_set_sensitive(no_button, FALSE);
  gtk_widget_set_can_default(no_button, TRUE);
  g_signal_connect(G_OBJECT(no_button), "clicked", G_CALLBACK(on_no_clicked),  NULL);
  
  statusbar = gtk_statusbar_new();  
  gtk_box_pack_start(GTK_BOX(vbox1), statusbar, TRUE, TRUE, 0);
  
  gtk_widget_grab_default(submit_button);
  gtk_widget_show_all(mainwin);

  return mainwin;
}


void 
set_timer_pct (GtkWidget *mainwin,
               gfloat     pct)
{
   gtk_entry_set_progress_fraction(GTK_ENTRY(answer_entry), pct);
}


void
disable_all_input_widgets (GtkWidget *mainwin)
{
   g_assert(submit_button != NULL);
   gtk_widget_set_sensitive(submit_button, FALSE);
   gtk_widget_grab_default(submit_button);

   g_assert(pause_button != NULL);
   gtk_widget_set_sensitive(pause_button, FALSE);
   gtk_widget_grab_focus(pause_button);

   g_assert(yes_button != NULL);
   gtk_widget_set_sensitive(yes_button, FALSE);

   g_assert(no_button != NULL);
   gtk_widget_set_sensitive(no_button, FALSE);

   g_assert(answer_entry != NULL);
   gtk_entry_set_text(GTK_ENTRY(answer_entry), "");
   gtk_widget_grab_focus(answer_entry);
}


void
reset_input_widgets (GtkWidget *mainwin)
{
   g_assert(submit_button != NULL);
   gtk_widget_set_sensitive(submit_button, TRUE);
   gtk_widget_grab_default(submit_button);

   g_assert(pause_button != NULL);
   gtk_widget_set_sensitive(pause_button, TRUE);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(pause_button), FALSE);
   gtk_widget_grab_focus(pause_button);

   g_assert(yes_button != NULL);
   gtk_widget_set_sensitive(yes_button, FALSE);

   g_assert(no_button != NULL);
   gtk_widget_set_sensitive(no_button, FALSE);

   g_assert(answer_entry != NULL);
   gtk_entry_set_text(GTK_ENTRY(answer_entry), "");
   gtk_widget_grab_focus(answer_entry);
}


void
display_rounds_left (GtkWidget *mainwin,
                     gint       rounds)
{
   gchar *str;
   guint contextid;
   
   str = g_strdup_printf(_("Rounds Left: %d"), rounds);
   
   contextid = gtk_statusbar_get_context_id(GTK_STATUSBAR(statusbar), (const gchar *)str);
   
   gtk_statusbar_push(GTK_STATUSBAR(statusbar), contextid, (const gchar *)str);
   
   g_free(str);
}


void
yesno_activate (GtkWidget *mainwin)
{
   g_assert(submit_button != NULL);
   gtk_widget_set_sensitive(submit_button, FALSE);

   g_assert(yes_button != NULL);
   gtk_widget_set_sensitive(yes_button, TRUE);

   g_assert(no_button != NULL);
   gtk_widget_set_sensitive(no_button, TRUE);
}
